@@ -21,7 +21,7 @@ from mercurial import (
     node,
     obsolete,
     pycompat,
-    registrar,
+    rewriteutil,
     scmutil,
     util,
 )
@@ -54,14 +54,6 @@ TROUBLES = {
     r'PHASEDIVERGENT': b'phase-divergent',
 }
 
-# XXX: Better detection of property cache
-if r'predecessors' not in dir(obsolete.obsstore):
-    @property
-    def predecessors(self):
-        return self.precursors
-
-    obsolete.obsstore.predecessors = predecessors
-
 def memfilectx(repo, ctx, fctx, flags, copied, path):
     # XXX Would it be better at the module level?
     varnames = context.memfilectx.__init__.__code__.co_varnames  # pytype: disable=attribute-error
@@ -337,15 +329,6 @@ fixupstreamed = util.safehasattr(scmutil, 'movedirstate')
 if not fixupstreamed:
     copiesmod._fullcopytracing = fixedcopytracing
 
-# help category compatibility
-# hg <= 4.7 (c303d65d2e34)
-def helpcategorykwargs(categoryname):
-    """Backwards-compatible specification of the helpategory argument."""
-    category = getattr(registrar.command, categoryname, None)
-    if not category:
-        return {}
-    return {'helpcategory': category}
-
 # nodemap.get and index.[has_node|rev|get_rev]
 # hg <= 5.2 (02802fa87b74)
 def getgetrev(cl):
@@ -355,9 +338,15 @@ def getgetrev(cl):
     return cl.nodemap.get
 
 @contextlib.contextmanager
-def parentchange(repo):
+def changing_parents(repo):
+    if util.safehasattr(repo.dirstate, 'changing_parents'):
+        changing_parents = repo.dirstate.changing_parents(repo)
+    else:
+        # hg <= 6.3 (7a8bfc05b691)
+        changing_parents = repo.dirstate.parentchange()
     try:
-        yield
+        with changing_parents:
+            yield
     finally:
         # hg <= 5.2 (85c4cd73996b)
         if util.safehasattr(repo, '_quick_access_changeid_invalidate'):
@@ -387,21 +376,6 @@ else:
     def clean_update(ctx):
         hg.updaterepo(ctx.repo(), ctx.node(), overwrite=True)
 
-def cleanupnodes(repo, replacements, operation, moves=None, metadata=None):
-    # Use this condition as a proxy since the commit we care about
-    # (b99903534e06) didn't change any signatures.
-    if util.safehasattr(scmutil, 'nullrev'):
-        fixedreplacements = replacements
-    else:
-        # hg <= 4.7 (b99903534e06)
-        fixedreplacements = {}
-        for oldnodes, newnodes in replacements.items():
-            for oldnode in oldnodes:
-                fixedreplacements[oldnode] = newnodes
-
-    scmutil.cleanupnodes(repo, replacements=fixedreplacements, operation=operation,
-                         moves=moves, metadata=metadata)
-
 if util.safehasattr(cmdutil, 'format_changeset_summary'):
     def format_changeset_summary_fn(ui, repo, command, default_spec):
         def show(ctx):
@@ -424,17 +398,12 @@ else:
         def to_display(name):
             return pycompat.sysbytes(name).replace(b'_', b'-')
 
-        if util.safehasattr(error, 'InputError'):
-            err = error.InputError
-        else:
-            # hg <= 5.6 (8d72e29ad1e0)
-            err = error.Abort
         previous = None
         for x in args:
             if opts.get(x):
                 if previous:
-                    raise err(_(b'cannot specify both --%s and --%s')
-                              % (to_display(previous), to_display(x)))
+                    raise InputError(_(b'cannot specify both --%s and --%s')
+                                     % (to_display(previous), to_display(x)))
                 previous = x
         return previous
 
@@ -561,6 +530,15 @@ if not markdirtyfixed:
 
     context.overlayworkingctx._markdirty = fixedmarkdirty
 
+def setbranch(repo, branch):
+    # this attribute was introduced at about the same time dirstate.setbranch()
+    # was modified
+    # hg <= 6.3 (e9379b55ed80)
+    if util.safehasattr(dirstate, 'requires_changing_files_or_status'):
+        repo.dirstate.setbranch(branch, repo.currenttransaction())
+    else:
+        repo.dirstate.setbranch(branch)
+
 if util.safehasattr(dirstate.dirstate, 'get_entry'):
     def dirchanges(dirstate):
         return [
@@ -570,3 +548,38 @@ else:
     # hg <= 5.9 (dcd97b082b3b)
     def dirchanges(dirstate):
         return [f for f in dirstate if dirstate[f] != b'n']
+
+if util.safehasattr(error, 'InputError'):
+    InputError = error.InputError
+else:
+    # hg <= 5.6 (8d72e29ad1e0)
+    InputError = error.Abort
+
+if util.safehasattr(error, 'StateError'):
+    StateError = error.StateError
+else:
+    # hg <= 5.6 (527ce85c2e60)
+    StateError = error.Abort
+
+try:
+    retained_extras_on_rebase = rewriteutil.retained_extras_on_rebase
+    preserve_extras_on_rebase = rewriteutil.preserve_extras_on_rebase
+except AttributeError:
+    # hg <= 6.4 (cbcbf63b6dbf)
+    retained_extras_on_rebase = {
+        b'source',
+        b'intermediate-source',
+    }
+
+    def preserve_extras_on_rebase(old_ctx, new_extra):
+        """preserve the relevant `extra` entries from old_ctx on rebase-like operations
+        """
+        old_extra = old_ctx.extra()
+        for key in retained_extras_on_rebase:
+            value = old_extra.get(key)
+            if value is not None:
+                new_extra[key] = value
+
+    # give other extensions an opportunity to collaborate
+    rewriteutil.retained_extras_on_rebase = retained_extras_on_rebase
+    rewriteutil.preserve_extras_on_rebase = preserve_extras_on_rebase
