@@ -28,3 +28,12 @@ def has_flake8():
 def has_check_manifest():
     return hghave.matchoutput('check-manifest --version 2>&1',
                               br'check-manifest version')
+
+@hghave.check("default-cg3", "changegroup3 by default")
+def has_default_changegroup3():
+    from mercurial import configitems
+
+    try:
+        return configitems.coreitems[b'experimental'][b'changegroup3'].default
+    except (AttributeError, KeyError):
+        return False
